@@ -20,21 +20,24 @@ logger = logging.getLogger("app")
 
 from enrichsdk.utils import get_lineage_of_query
 
-class CustomEncoder(json.JSONEncoder):
 
+class CustomEncoder(json.JSONEncoder):
     def default(self, obj):
         try:
             return super().default(obj)
         except:
             return str(obj)
 
+
 def get_yesterday():
     yesterday = date.today() + timedelta(days=-1)
     return yesterday.isoformat()
 
+
 def get_today():
     return date.today().isoformat()
 
+
 class InMemoryQueryExecutorBase(Compute):
     """
     Base class for an InMemory QueryExecutor transform. This is useful
@@ -62,22 +65,16 @@ class InMemoryQueryExecutorBase(Compute):
      Specs
 
     """
+
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self.name = "QueryExecutorBase"
         self.description = "Execute queries against backends"
         self.testdata = {
-            'data_root': os.path.join(os.environ['ENRICH_TEST'],
-                                      self.name),
-            'statedir': os.path.join(os.environ['ENRICH_TEST'],
-                                     self.name, 'state'),
-            'conf': {
-                'args': {
-
-                }
-            },
-            'data': {
-            }
+            "data_root": os.path.join(os.environ["ENRICH_TEST"], self.name),
+            "statedir": os.path.join(os.environ["ENRICH_TEST"], self.name, "state"),
+            "conf": {"args": {}},
+            "data": {},
         }
 
     @classmethod
@@ -90,36 +87,36 @@ class InMemoryQueryExecutorBase(Compute):
         """
         args = super().preload_clean_args(args)
 
-        if (("start_date" not in args) or
-            ("end_date" not in args)):
+        if ("start_date" not in args) or ("end_date" not in args):
             raise Exception("Start or end of timeframe missing")
 
         try:
-            start = dateparser.parse(args['start_date']).date()
-            args['start_date'] = start
-            end = dateparser.parse(args['end_date']).date()
-            args['end_date'] = end
+            start = dateparser.parse(args["start_date"]).date()
+            args["start_date"] = start
+            end = dateparser.parse(args["end_date"]).date()
+            args["end_date"] = end
         except:
-            logger.exception("Invalid start_date or end_date",
-                             extra={
-                                 'transform': self.name
-                             })
+            logger.exception(
+                "Invalid start_date or end_date", extra={"transform": self.name}
+            )
             raise Exception("Invalid start/end datetime specified")
 
-        if (('targets' not in args) or
-            (not isinstance(args['targets'], str)) or
-            (len(args['targets']) == 0)):
+        if (
+            ("targets" not in args)
+            or (not isinstance(args["targets"], str))
+            or (len(args["targets"]) == 0)
+        ):
             raise Exception("Invalid list of query names specified")
 
         # Include force
-        force = str(args['force']).lower().strip()
-        force = force == 'true'
-        args['force'] = force
+        force = str(args["force"]).lower().strip()
+        force = force == "true"
+        args["force"] = force
 
         # Clean the list of names...
-        targets = args['targets'].split(",")
+        targets = args["targets"].split(",")
         targets = [n.strip() for n in targets if len(n.strip()) > 0]
-        args['targets'] = [n for n in targets if len(n) > 0]
+        args["targets"] = [n for n in targets if len(n) > 0]
 
         return args
 
@@ -133,16 +130,16 @@ class InMemoryQueryExecutorBase(Compute):
         specs = self.get_sql_specs()
 
         # Compute the targets
-        targets = ['all'] #default
+        targets = ["all"]  # default
         for s in specs:
-            if not s.get('enable', True):
+            if not s.get("enable", True):
                 continue
-            categories = s.get('categories', [])
+            categories = s.get("categories", [])
             for c in categories:
                 if c not in targets:
                     targets.append(c)
         for s in specs:
-            name = s['name']
+            name = s["name"]
             if name not in targets:
                 targets.append(name)
         targets = "|".join(targets)
@@ -154,26 +151,26 @@ class InMemoryQueryExecutorBase(Compute):
                 "name": "targets",
                 "description": f"What all to run. Specify multiple with comma separating names ({targets})",
                 "default": "all",
-                "required": False
+                "required": False,
             },
             {
                 "name": "force",
                 "description": "Force execution",
                 "default": "False",
-                "required": False
+                "required": False,
             },
             {
                 "name": "start_date",
                 "description": "Start of the time window",
                 "default": get_yesterday(),
-                "required": True
+                "required": True,
             },
             {
                 "name": "end_date",
                 "description": "End of the time window",
                 "default": get_today(),
-                "required": True
-            }
+                "required": True,
+            },
         ] + remaining
 
     def update_frame(self, name, engine, sql, df, dependencies=[]):
@@ -191,27 +188,24 @@ class InMemoryQueryExecutorBase(Compute):
         # Insert extra dependencies
         dependencies += get_lineage_of_query(engine, sql)
 
-        lineage = {
-            "type": "lineage",
-            "dependencies": dependencies
-        }
+        lineage = {"type": "lineage", "dependencies": dependencies}
 
         # Generate column information...
         columns = self.get_column_metadata(name, df)
 
         ## => Gather the update parameters
         updated_detail = {
-            'df': None,
-            'description': f"Output for query {name}",
-            'transform': self.name,
-            'frametype': 'pandas',
-            'params': [
+            "df": None,
+            "description": f"Output for query {name}",
+            "transform": self.name,
+            "frametype": "pandas",
+            "params": [
                 {
-                    'type': 'compute',
-                    'columns': columns,
+                    "type": "compute",
+                    "columns": columns,
                 },
-                lineage
-            ]
+                lineage,
+            ],
         }
 
         # Dump it into the shared state
@@ -219,35 +213,49 @@ class InMemoryQueryExecutorBase(Compute):
 
     def generate_paramsets(self, spec, start_date, end_date):
 
-        duration = spec.get('paramsets_duration', 'full')
-        window   = spec.get('paramsets_window', 0)
+        duration = spec.get("paramsets_duration", "full")
+        window = spec.get("paramsets_window", 0)
 
         if isinstance(start_date, str):
             start_date = parser.parse(start_date).date()
 
         if isinstance(end_date, str):
-            end_date   = parser.parse(end_date).date()
+            end_date = parser.parse(end_date).date()
 
-        if duration not in ['day', 'full']:
+        if duration not in ["day", "full"]:
             raise Exception("Only day and full are supported. full is default")
 
-        if duration == 'full':
+        if duration == "full":
             return [
                 {
-                    'start_date': start_date.isoformat(),
-                    'end_date': end_date.isoformat(),
-                    'start_date_minus7': (start_date + relativedelta.relativedelta(days=-7)).isoformat(),
-                    'start_date_minus1': (start_date + relativedelta.relativedelta(days=-1)).isoformat(),
-                    'start_date_minus6_months': (start_date + relativedelta.relativedelta(months=-6)).isoformat(),
-                    'end_date_plus1': (end_date + relativedelta.relativedelta(days=1)).isoformat(),
-                    'end_date_minus1': (end_date + relativedelta.relativedelta(days=-1)).isoformat(),
-                    'end_date_minus7': (end_date + relativedelta.relativedelta(days=-7)).isoformat(),
-                    'end_date_minus6_months': (end_date + relativedelta.relativedelta(months=-6)).isoformat(),
+                    "start_date": start_date.isoformat(),
+                    "end_date": end_date.isoformat(),
+                    "start_date_minus7": (
+                        start_date + relativedelta.relativedelta(days=-7)
+                    ).isoformat(),
+                    "start_date_minus1": (
+                        start_date + relativedelta.relativedelta(days=-1)
+                    ).isoformat(),
+                    "start_date_minus6_months": (
+                        start_date + relativedelta.relativedelta(months=-6)
+                    ).isoformat(),
+                    "end_date_plus1": (
+                        end_date + relativedelta.relativedelta(days=1)
+                    ).isoformat(),
+                    "end_date_minus1": (
+                        end_date + relativedelta.relativedelta(days=-1)
+                    ).isoformat(),
+                    "end_date_minus7": (
+                        end_date + relativedelta.relativedelta(days=-7)
+                    ).isoformat(),
+                    "end_date_minus6_months": (
+                        end_date + relativedelta.relativedelta(months=-6)
+                    ).isoformat(),
                 }
             ]
 
         # There may be more. So leaving it here...
-        if duration == 'day':
+        if duration == "day":
 
             # Parse the dates
             if start_date > end_date:
@@ -262,17 +270,35 @@ class InMemoryQueryExecutorBase(Compute):
             paramsets = []
             curr_date = start_date
             while curr_date <= end_date:
-                paramsets.append({
-                    'start_date': curr_date.isoformat(),
-                    'end_date': (curr_date + relativedelta.relativedelta(days=window)).isoformat(),
-                    'start_date_minus7': (curr_date + relativedelta.relativedelta(days=-7)).isoformat(),
-                    'start_date_minus1': (curr_date + relativedelta.relativedelta(days=-1)).isoformat(),
-                    'start_date_plus1': (curr_date + relativedelta.relativedelta(days=1)).isoformat(),
-                    'end_date_minus1': (curr_date + relativedelta.relativedelta(days=-1)).isoformat(),
-                    'end_date_plus1': (curr_date + relativedelta.relativedelta(days=1)).isoformat(),
-                    'end_date_minus7': (curr_date + relativedelta.relativedelta(days=-7)).isoformat(),
-                    'end_date_minus6_months': (curr_date + relativedelta.relativedelta(months=-6)).isoformat(),
-                })
+                paramsets.append(
+                    {
+                        "start_date": curr_date.isoformat(),
+                        "end_date": (
+                            curr_date + relativedelta.relativedelta(days=window)
+                        ).isoformat(),
+                        "start_date_minus7": (
+                            curr_date + relativedelta.relativedelta(days=-7)
+                        ).isoformat(),
+                        "start_date_minus1": (
+                            curr_date + relativedelta.relativedelta(days=-1)
+                        ).isoformat(),
+                        "start_date_plus1": (
+                            curr_date + relativedelta.relativedelta(days=1)
+                        ).isoformat(),
+                        "end_date_minus1": (
+                            curr_date + relativedelta.relativedelta(days=-1)
+                        ).isoformat(),
+                        "end_date_plus1": (
+                            curr_date + relativedelta.relativedelta(days=1)
+                        ).isoformat(),
+                        "end_date_minus7": (
+                            curr_date + relativedelta.relativedelta(days=-7)
+                        ).isoformat(),
+                        "end_date_minus6_months": (
+                            curr_date + relativedelta.relativedelta(months=-6)
+                        ).isoformat(),
+                    }
+                )
                 curr_date += relativedelta.relativedelta(days=1)
 
             return paramsets
@@ -349,7 +375,7 @@ class InMemoryQueryExecutorBase(Compute):
 
         files = glob.glob(sqldir + "/*.sql")
         for f in files:
-            name = os.path.basename(f).replace(".sql","")
+            name = os.path.basename(f).replace(".sql", "")
             sql = open(f).read()
 
             # Specify the split in the SQL itself..
@@ -366,12 +392,9 @@ class InMemoryQueryExecutorBase(Compute):
             if match is not None:
                 engine = match.group(1).strip()
 
-            specs.append({
-                'name': name,
-                'sql': sql,
-                'segment': segment,
-                'engine': engine
-            })
+            specs.append(
+                {"name": name, "sql": sql, "segment": segment, "engine": engine}
+            )
 
         return specs
 
@@ -379,55 +402,54 @@ class InMemoryQueryExecutorBase(Compute):
         """
         Run the computation and update the state
         """
-        logger.debug("Start execution",
-                     extra=self.config.get_extra({
-                         'transform': self.name
-                     }))
+        logger.debug(
+            "Start execution", extra=self.config.get_extra({"transform": self.name})
+        )
 
         # Will be used in other places..
         self.state = state
 
         # List of specification names
-        targets = self.args['targets']
+        targets = self.args["targets"]
 
         # Get specs..
         specs = self.get_sql_specs()
 
-        logger.debug(f"Specs found: {len(specs)}",
-                     extra={
-                         'transform': self.name
-                     })
+        logger.debug(f"Specs found: {len(specs)}", extra={"transform": self.name})
         # Now iterate through the specs.
         for spec in specs:
             try:
 
-                name = spec['name']
-                categories = spec.get('categories', ['all'])
-                table = spec.get('table',name)
-                cond = spec.get('cond','')
-                retries = spec.get('retries', 1)
+                name = spec["name"]
+                categories = spec.get("categories", ["all"])
+                table = spec.get("table", name)
+                cond = spec.get("cond", "")
+                retries = spec.get("retries", 1)
 
                 # To take care of the logging in case of exception
                 msg = f"Name: {name}\n"
 
                 # Check if this has been requested?
-                if (all([c not in targets for c in categories]) and
-                    (name not in targets)):
+                if all([c not in targets for c in categories]) and (
+                    name not in targets
+                ):
                     continue
 
-                logger.debug(f"Executing {spec['name']}",
-                             extra={
-                                 'transform': self.name,
-                                 'data': json.dumps(spec, indent=4, cls=CustomEncoder)
-                     })
+                logger.debug(
+                    f"Executing {spec['name']}",
+                    extra={
+                        "transform": self.name,
+                        "data": json.dumps(spec, indent=4, cls=CustomEncoder),
+                    },
+                )
 
-                sql_template = spec['sql']
+                sql_template = spec["sql"]
 
-                paramsets = self.generate_paramsets(spec,
-                                                    self.args['start_date'],
-                                                    self.args['end_date'])
+                paramsets = self.generate_paramsets(
+                    spec, self.args["start_date"], self.args["end_date"]
+                )
 
-                for params in  paramsets:
+                for params in paramsets:
 
                     status = []
 
@@ -441,7 +463,7 @@ class InMemoryQueryExecutorBase(Compute):
                     # Get the engine for a given spec
                     engine = self.get_engine(spec)
 
-                    segmentcol = spec.get('segment', None)
+                    segmentcol = spec.get("segment", None)
 
                     tryno = 1
                     while True:
@@ -452,11 +474,10 @@ class InMemoryQueryExecutorBase(Compute):
                             df = pd.read_sql(sql, engine)
                             break
                         except:
-                            logger.exception(f"Failed Query: {name} (try {tryno})",
-                                 extra={
-                                     'transform': self.name,
-                                     'data': msg
-                                 })
+                            logger.exception(
+                                f"Failed Query: {name} (try {tryno})",
+                                extra={"transform": self.name, "data": msg},
+                            )
                         tryno += 1
                         time.sleep(30)
 
@@ -466,22 +487,21 @@ class InMemoryQueryExecutorBase(Compute):
                     msg += f"Segment: {segmentcol} (Initial split)\n"
                     msg += "Records: {}\n".format(df.shape[0])
                     msg += "Columns: {}\n".format(", ".join(df.columns))
-                    msg += "Dtypes: " + df.dtypes.to_string() + '\n'
+                    msg += "Dtypes: " + df.dtypes.to_string() + "\n"
 
                     if len(df) == 0:
                         # no data returned...
-                        logger.warning(f"Completed {name} {params['start_date']} No data",
-                                     extra={
-                                         'transform': self.name,
-                                         'data': msg
-                                 })
+                        logger.warning(
+                            f"Completed {name} {params['start_date']} No data",
+                            extra={"transform": self.name, "data": msg},
+                        )
                         continue
 
                     # First gather a map of segments
                     filemap = {}
                     if segmentcol is None:
                         # Whole thing is one segment
-                        filemap['complete'] = df
+                        filemap["complete"] = df
                     elif isinstance(segmentcol, str):
                         # Split by column name...
                         segments = list(df[segmentcol].unique())
@@ -509,10 +529,12 @@ class InMemoryQueryExecutorBase(Compute):
 
                         # Clean the output data...
                         try:
-                            if 'clean' in spec:
-                                callback = spec['clean']['callback']
+                            if "clean" in spec:
+                                callback = spec["clean"]["callback"]
                                 if callable(callback):
-                                    clean_msg, df1 = callback(self, segmentcol, segment, df1, spec)
+                                    clean_msg, df1 = callback(
+                                        self, segmentcol, segment, df1, spec
+                                    )
                                     if len(clean_msg) > 0:
                                         msg += f"[{segment}] " + clean_msg + "\n"
                         except Exception as e:
@@ -522,11 +544,13 @@ class InMemoryQueryExecutorBase(Compute):
                         # Store in database...
                         try:
                             extra_dependencies = []
-                            if 'store' in spec:
+                            if "store" in spec:
                                 # Separate storage handler..
-                                callback = spec['store']['callback']
+                                callback = spec["store"]["callback"]
                                 if callable(callback):
-                                    store_msg, store_dependencies = callback(self, segmentcol, segment, df1, spec)
+                                    store_msg, store_dependencies = callback(
+                                        self, segmentcol, segment, df1, spec
+                                    )
                                     if len(store_msg) > 0:
                                         msg += f"[{segment}] " + store_msg + "\n"
                                     extra_dependencies += store_dependencies
@@ -538,40 +562,40 @@ class InMemoryQueryExecutorBase(Compute):
                         try:
 
                             # Store in s3 etc.
-                            store_msg, store_dependencies  = self.store(segmentcol, segment, df1, spec)
+                            store_msg, store_dependencies = self.store(
+                                segmentcol, segment, df1, spec
+                            )
                             if len(store_msg) > 0:
                                 msg += f"[{segment}] " + store_msg
                             extra_dependencies += store_dependencies
 
                             # update lineage
-                            self.update_frame(name, engine, sql, df1, extra_dependencies)
+                            self.update_frame(
+                                name, engine, sql, df1, extra_dependencies
+                            )
 
                         except Exception as e:
                             msg += "[{}] Exception {}\n".format(segment, str(e))
 
-                    logger.debug(f"Completed {name} {params['start_date']}",
-                                 extra={
-                                     'transform': self.name,
-                                     'data': msg
-                                 })
+                    logger.debug(
+                        f"Completed {name} {params['start_date']}",
+                        extra={"transform": self.name, "data": msg},
+                    )
             except:
                 # Exception for each spec.
-                logger.exception(f"Unable to run query: {name}",
-                                 extra={
-                                     'transform': self.name,
-                                     'data': msg
-                                 })
+                logger.exception(
+                    f"Unable to run query: {name}",
+                    extra={"transform": self.name, "data": msg},
+                )
                 msg = ""
                 continue
 
-
         self.add_marker(state)
 
         # Done
-        logger.debug("Complete execution",
-                     extra=self.config.get_extra({
-                         'transform': self.name
-                     }))
+        logger.debug(
+            "Complete execution", extra=self.config.get_extra({"transform": self.name})
+        )
 
         ###########################################
         # => Return
