@@ -19,35 +19,38 @@ from ..lib import read_siteconf, read_versionmap
 from ..lib.context import Context
 import logging
 
+
 class CustomEncoder(json.JSONEncoder):
     def default(self, obj):
         try:
-            if 'DataFrame' in obj.__class__.__name__:
+            if "DataFrame" in obj.__class__.__name__:
                 return "<dataframe>"
             return super().default(obj)
         except:
             return str(obj)
 
-logger = logging.getLogger('app')
+
+logger = logging.getLogger("app")
+
 
 class MockRun(object):
-
     def __init__(self, *args, **kwargs):
-        self.state = kwargs.pop('state')
+        self.state = kwargs.pop("state")
         self.files = []
 
     def load_files(self):
         frames = self.state.get_frame_list()
         for f in frames:
             detail = self.state.get_frame(f)
-            detail['name'] = f
+            detail["name"] = f
             self.files.append(detail)
 
     def list_files(self):
-        return [f['name'] for f in self.files]
+        return [f["name"] for f in self.files]
 
     def get_file_hash(self):
-        return { f['name']: f for f in self.files}
+        return {f["name"]: f for f in self.files}
+
 
 class MockRunManager(object):
     def __init__(self, *args, **kwargs):
@@ -55,17 +58,17 @@ class MockRunManager(object):
 
     def load_runs(self, runs, state):
         for r in runs:
-            status = r['status']
-            runid = r['runid']
+            status = r["status"]
+            runid = r["runid"]
             run = MockRun(state=state)
-            run.status = r['status']
-            run.runid = r['runid']
-            run.start_time = date_parser.parse(r['start_time'])
-            run.end_time = date_parser.parse(r['end_time'])
+            run.status = r["status"]
+            run.runid = r["runid"]
+            run.start_time = date_parser.parse(r["start_time"])
+            run.end_time = date_parser.parse(r["end_time"])
 
             self.runs.append(run)
 
-    def get_last_run(self, status='success'):
+    def get_last_run(self, status="success"):
         for r in self.runs:
             if r.status == status:
                 return r
@@ -74,123 +77,121 @@ class MockRunManager(object):
     def load(self):
         pass
 
-class MockState(object):
 
+class MockState(object):
     def __init__(self, *args, **kwargs):
         self.name = "ExperimentalPipeline"
         self.runid = "exp-pipeline-20180922-192321"
-        self.config = kwargs.pop('config', None)
+        self.config = kwargs.pop("config", None)
         self.state = {}
         self.notes = []
         self.performance_notes = []
         self.expectations = []
-        self.notifications = {
-            'email': []
-        }
+        self.notifications = {"email": []}
         user = getpass.getuser()
         self.pid = os.getpid()
         self.start_time = datetime.now()
         self.end_time = datetime.now()
         self.stats = {
-            'platform': {
-                'node': platform.node(),
-                'os': platform.system(),
-                'release': platform.release(),
-                'processor': platform.processor(),
-                'python': platform.python_version(),
-                'distribution': distro.linux_distribution()
+            "platform": {
+                "node": platform.node(),
+                "os": platform.system(),
+                "release": platform.release(),
+                "processor": platform.processor(),
+                "python": platform.python_version(),
+                "distribution": distro.linux_distribution(),
             },
-            'user': user
+            "user": user,
         }
 
     def make_note(self, text):
         self.notes.append(text)
 
-    def make_performance_note(self, name,
-                              description,
-                              condition,
-                              priority=1,
-                              message=""):
-        self.performance_notes.append({
-            'name': name,
-            'description': description,
-            'condition': condition,
-            'priority': priority,
-            'message': message
-        })
+    def make_performance_note(
+        self, name, description, condition, priority=1, message=""
+    ):
+        self.performance_notes.append(
+            {
+                "name": name,
+                "description": description,
+                "condition": condition,
+                "priority": priority,
+                "message": message,
+            }
+        )
 
     def update_frame(self, framename, details, create=False):
 
-        details['name'] = framename
+        details["name"] = framename
 
-        for d in ['df', 'transform']:
+        for d in ["df", "transform"]:
             if d not in details:
                 raise Exception("Missing frame detail: {}".format(d))
-        if 'frametype' not in details:
-            details['frametype'] = 'pandas'
+        if "frametype" not in details:
+            details["frametype"] = "pandas"
 
-        if 'description' not in details:
-            details['description'] = ''
+        if "description" not in details:
+            details["description"] = ""
 
-        if 'category' not in details:
-            details['category'] = 'default'
+        if "category" not in details:
+            details["category"] = "default"
 
-        if 'stages' not in details:
-            details['stages'] = []
-        if isinstance(details['transform'], str):
-            details['stages'].append(details['transform'])
+        if "stages" not in details:
+            details["stages"] = []
+        if isinstance(details["transform"], str):
+            details["stages"].append(details["transform"])
         else:
-            details['stages'].extend(details['transform'])
+            details["stages"].extend(details["transform"])
 
-        if 'params' not in details:
-            details['params'] = []
-        elif isinstance(details['params'], dict):
-            details['params'] = [details['params']]
-        elif isinstance(details['params'], list):
-            for p in details['params']:
+        if "params" not in details:
+            details["params"] = []
+        elif isinstance(details["params"], dict):
+            details["params"] = [details["params"]]
+        elif isinstance(details["params"], list):
+            for p in details["params"]:
                 if not isinstance(p, dict):
                     raise Exception("Invalid params. List items should be dicts")
         else:
             raise Exception("Invalid params. Expected list or dict")
 
-        if 'history' not in details:
-            details['history'] = []
-        elif isinstance(details['history'], dict):
-            details['history'] = [details['history']]
+        if "history" not in details:
+            details["history"] = []
+        elif isinstance(details["history"], dict):
+            details["history"] = [details["history"]]
 
         if framename not in self.state:
             self.state[framename] = details
         else:
             frame = self.state[framename]
-            frame['df'] = details['df']
-            frame['source'] = details['transform']
-            frame['stages'].append(details['transform'])
-            frame['history'].extend(details['history'])
+            frame["df"] = details["df"]
+            frame["source"] = details["transform"]
+            frame["stages"].append(details["transform"])
+            frame["history"].extend(details["history"])
 
             # Handle description
-            if ((isinstance(details['description'], str)) and
-                (len(details['description']) > 0)):
-                if frame['description'] != '':
-                    frame['description'] += ". "
-                frame['description'] += details['description']
+            if (isinstance(details["description"], str)) and (
+                len(details["description"]) > 0
+            ):
+                if frame["description"] != "":
+                    frame["description"] += ". "
+                frame["description"] += details["description"]
 
             # Incorporate params
-            if 'params' in details:
-                if isinstance(details['params'], dict):
-                    frame['params'].insert(0, details['params'])
-                elif isinstance(details['params'], list):
-                    for p in details['params']:
+            if "params" in details:
+                if isinstance(details["params"], dict):
+                    frame["params"].insert(0, details["params"])
+                elif isinstance(details["params"], list):
+                    for p in details["params"]:
                         if not isinstance(p, dict):
                             raise Exception("Invalid params type")
                         if len(p) == 0:
                             continue
-                        frame['params'].append(p)
+                        frame["params"].append(p)
                 else:
                     raise Exception("Unknown params")
 
-            if ((frame['category'] == 'default') and
-                (details['category'] != 'default')):
-                frame['category'] = details['category']
+            if (frame["category"] == "default") and (details["category"] != "default"):
+                frame["category"] = details["category"]
 
     def has_frame(self, framename):
         return framename in self.state
@@ -207,44 +208,47 @@ class MockState(object):
 
         frames = []
         for f in allframes:
-            if self.state[f]['category'] == category:
+            if self.state[f]["category"] == category:
                 frames.append(f)
 
         return frames
 
     def reached_stage(self, frame, name):
-        return ((frame in self.state) and
-                (name in self.state[frame]['stages']))
+        return (frame in self.state) and (name in self.state[frame]["stages"])
 
     def add_expectations(self, transform, frame, expectations):
         if not isinstance(expectations, list):
             raise Exception("Expectations should be a list")
 
         for e in expectations:
-            for col in ['expectation', 'description','passed']:
+            for col in ["expectation", "description", "passed"]:
                 if col not in e:
-                    raise Exception("Expectation results are missing col {}".format(col))
+                    raise Exception(
+                        "Expectation results are missing col {}".format(col)
+                    )
 
         # => Insert metadata and store
         expanded = []
         for e in expectations:
             e = copy.copy(e)
-            e.update({
-                'version': 'v1',
-                'timestamp': self.config.now().isoformat(),
-                'frame': frame,
-                'meta': {
-                    'node': self.stats['platform']['node'],
-                    'pipeline': self.name,
-                    'transform': transform.name,
-                    'runid': self.runid,
-                    'start_time': self.start_time.isoformat(),
+            e.update(
+                {
+                    "version": "v1",
+                    "timestamp": self.config.now().isoformat(),
+                    "frame": frame,
+                    "meta": {
+                        "node": self.stats["platform"]["node"],
+                        "pipeline": self.name,
+                        "transform": transform.name,
+                        "runid": self.runid,
+                        "start_time": self.start_time.isoformat(),
+                    },
                 }
-            })
+            )
             self.expectations.append(e)
 
-class MockConfig(object):
 
+class MockConfig(object):
     def __init__(self, context={}, *args, **kwargs):
 
         # Initialize the context
@@ -255,40 +259,36 @@ class MockConfig(object):
 
         self.name = "TestConf"
         self.state = MockState(config=self)
-        self.runid = 'run-2015021'
+        self.runid = "run-2015021"
         self.description = "Experimental pipeline for testing transforms"
         self.context = context
         self.runmanager = MockRunManager()
-        self.dataframes = {
-            'pandas': PandasDataFrame(),
-            'dict': DictDataFrame()
-        }
+        self.dataframes = {"pandas": PandasDataFrame(), "dict": DictDataFrame()}
         self.tz = pytz.timezone("Asia/Kolkata")
         self.ts = datetime.now(self.tz).replace(microsecond=0).isoformat()
         self.enable_extra_args = True
         self.readonly = False
-        self.data_root = self.context['ENRICH_TEST']
-        self.customer_root = self.context['ENRICH_CUSTOMERS']
-        self.enrich_customers = self.context['ENRICH_CUSTOMERS']
-        self.enrich_root = self.context['ENRICH_ROOT']
-        self.enrich_run_root = self.context['ENRICH_ROOT']
+        self.data_root = self.context["ENRICH_TEST"]
+        self.customer_root = self.context["ENRICH_CUSTOMERS"]
+        self.enrich_customers = self.context["ENRICH_CUSTOMERS"]
+        self.enrich_root = self.context["ENRICH_ROOT"]
+        self.enrich_run_root = self.context["ENRICH_ROOT"]
         self.enrich_data_dir = self.data_root
         self.accessed_files = []
         self.supported_extra_args = []
-        self.usecase = {
-            'org': {
-                'name': 'Unknown',
-                'customer': 'unknown'
-            }
-        }
+        self.usecase = {"org": {"name": "Unknown", "customer": "unknown"}}
         # Add defaults
-        for attr in ['data_root', 'enrich_data_dir',
-                     'inputdir', 'outputdir', 'statedir']:
+        for attr in [
+            "data_root",
+            "enrich_data_dir",
+            "inputdir",
+            "outputdir",
+            "statedir",
+        ]:
             setattr(self, attr, self.data_root)
 
-
     def get_usecase(self):
-        return self.usecase.get("org",{}).get("name","Unknown")
+        return self.usecase.get("org", {}).get("name", "Unknown")
 
     def get_customer(self):
         return self.get_usecase()
@@ -298,11 +298,11 @@ class MockConfig(object):
         enrichjson = None
         enrich_customers = self.enrich_customers
 
-        if 'ENRICHJSON' in os.environ:
-            enrichjson = os.environ['ENRICHJSON']
+        if "ENRICHJSON" in os.environ:
+            enrichjson = os.environ["ENRICHJSON"]
             enrichjson = os.path.abspath(enrichjson)
-        elif 'ENRICHJSON' in self.context:
-            enrichjson = os.path.abspath(self.context['ENRICHJSON'])
+        elif "ENRICHJSON" in self.context:
+            enrichjson = os.path.abspath(self.context["ENRICHJSON"])
         elif os.path.exists(enrich_customers):
             # Try inferring from the location.
             enrichjson = None
@@ -311,13 +311,12 @@ class MockConfig(object):
                 c = os.path.realpath(os.path.join(enrich_customers, c))
                 if path.startswith(c):
                     relpath = os.path.relpath(path, start=c)
-                    enrichjson = os.path.join(c,
-                                              relpath.split(os.sep)[0],
-                                              'enrich.json')
+                    enrichjson = os.path.join(
+                        c, relpath.split(os.sep)[0], "enrich.json"
+                    )
                     break
 
-        if ((enrichjson is None) or
-            (not os.path.exists(enrichjson))):
+        if (enrichjson is None) or (not os.path.exists(enrichjson)):
             logger.warning("Unable to find enrich.json")
             return {}
 
@@ -332,7 +331,7 @@ class MockConfig(object):
         return {}
 
     def configure(self, params):
-        for var in ['enable_extra_args']:
+        for var in ["enable_extra_args"]:
             if var in params:
                 setattr(self, var, params[var])
 
@@ -342,17 +341,16 @@ class MockConfig(object):
     def get_dataframe(self, name):
 
         if name == "pandas":
-            return self.dataframes['pandas']
-        elif name == 'dict':
-            return self.dataframes['dict']
+            return self.dataframes["pandas"]
+        elif name == "dict":
+            return self.dataframes["dict"]
         else:
             raise Exception("Unsupported dataframe type")
 
     def get_extra(self, extra={}):
         return extra
 
-    def get_file(self, template, abspath=True,
-                 create_dir=False, extra={}):
+    def get_file(self, template, abspath=True, create_dir=False, extra={}):
 
         if template is None:
             return None
@@ -364,21 +362,21 @@ class MockConfig(object):
 
         try:
             params = {
-                'name': self.name,
-                'outputdir': self.outputdir,
-                'output': self.outputdir,
-                'inputdir': self.inputdir,
-                'input': self.inputdir,
-                'statedir': self.statedir,
-                'state': self.statedir,
-                'runid': self.runid,
-                'ts': self.ts,
-                'date': dt,
-                'datetime': dtime,
-                'node': node,
-                'data_root': self.data_root,
-                'enrich_root': self.enrich_root,
-                'enrich_data_dir': self.enrich_data_dir,
+                "name": self.name,
+                "outputdir": self.outputdir,
+                "output": self.outputdir,
+                "inputdir": self.inputdir,
+                "input": self.inputdir,
+                "statedir": self.statedir,
+                "state": self.statedir,
+                "runid": self.runid,
+                "ts": self.ts,
+                "date": dt,
+                "datetime": dtime,
+                "node": node,
+                "data_root": self.data_root,
+                "enrich_root": self.enrich_root,
+                "enrich_data_dir": self.enrich_data_dir,
             }
             params.update(self.context)
             params.update(extra)
@@ -396,8 +394,8 @@ class MockConfig(object):
             except:
                 pass
 
-        transform = extra.get('transform', None)
-        meta = extra.get('meta', {})
+        transform = extra.get("transform", None)
+        meta = extra.get("meta", {})
         self.note_access(filename, transform, meta)
 
         return filename
@@ -410,9 +408,11 @@ class MockConfig(object):
         code = None
         stack = inspect.stack()
         for s in stack:
-            if ((".virtualenv" not in s.filename) and
-                ("enrichcli" not in s.filename) and
-                ("enrichsdk" not in s.filename)):
+            if (
+                (".virtualenv" not in s.filename)
+                and ("enrichcli" not in s.filename)
+                and ("enrichsdk" not in s.filename)
+            ):
                 code = s.filename
                 code = self.get_relative_path(code, what="enrich_root")
                 code += ":" + str(s.lineno)
@@ -424,24 +424,25 @@ class MockConfig(object):
             relpath = filename
 
         entry = {
-            'transform': transform.name if not isinstance(transform, str) else transform,
-            'filename': relpath,
-            'timestamp': datetime.now().isoformat(),
-            'source': code,
-            'meta': meta
+            "transform": transform.name
+            if not isinstance(transform, str)
+            else transform,
+            "filename": relpath,
+            "timestamp": datetime.now().isoformat(),
+            "source": code,
+            "meta": meta,
         }
 
         self.accessed_files.append(entry)
 
-
-    def get_relative_path(self, path, what='enrich_data_dir'):
+    def get_relative_path(self, path, what="enrich_data_dir"):
 
         paths = {
-            'enrich_root': self.enrich_root,
-            'enrich_data_dir': self.enrich_data_dir,
-            'outputdir': self.outputdir,
-            'output': self.outputdir,
-            'data_root': self.data_root,
+            "enrich_root": self.enrich_root,
+            "enrich_data_dir": self.enrich_data_dir,
+            "outputdir": self.outputdir,
+            "output": self.outputdir,
+            "data_root": self.data_root,
         }
         whatroot = paths[what]
         return os.path.relpath(path, whatroot)
@@ -449,20 +450,24 @@ class MockConfig(object):
     def load_test_state(self, testdata):
 
         app = {
-            'pipeline': self.name,
-            'description': self.description,
-            'data_root': self.data_root,
-            'customer_root': self.customer_root,
-            'runid': self.runid
+            "pipeline": self.name,
+            "description": self.description,
+            "data_root": self.data_root,
+            "customer_root": self.customer_root,
+            "runid": self.runid,
         }
 
         # Load siteconf
-        if 'siteconf' in testdata:
-            siteconf = read_siteconf(testdata['siteconf'],app=app, context=self.context)
-        elif 'siteconf' in self.context:
-            siteconf = read_siteconf(self.context['siteconf'],app=app, context=self.context)
+        if "siteconf" in testdata:
+            siteconf = read_siteconf(
+                testdata["siteconf"], app=app, context=self.context
+            )
+        elif "siteconf" in self.context:
+            siteconf = read_siteconf(
+                self.context["siteconf"], app=app, context=self.context
+            )
         else:
-            siteconf = read_siteconf(app=app,context=self.context)
+            siteconf = read_siteconf(app=app, context=self.context)
 
         if siteconf is None:
             raise Exception("Could not read siteconf")
@@ -470,8 +475,9 @@ class MockConfig(object):
         self.siteconf = siteconf
 
         # Load versionmap
-        filename = testdata.get('versionmap',
-                                self.get_file("%(ENRICH_ETC)s/versionmap.json"))
+        filename = testdata.get(
+            "versionmap", self.get_file("%(ENRICH_ETC)s/versionmap.json")
+        )
         timestamp, versionmap = read_versionmap(filename)
         self.versionmap = versionmap
 
@@ -479,75 +485,72 @@ class MockConfig(object):
         expand = os.path.expandvars
         defaultdir = self.get_file("%(ENRICH_TEST)s")
 
-        self.data_root = expand(testdata.get('data_root', defaultdir))
+        self.data_root = expand(testdata.get("data_root", defaultdir))
         self.enrich_data_dir = self.data_root
-        self.inputdir = expand(testdata.get('inputdir', self.data_root))
-        self.outputdir = expand(testdata.get('outputdir', self.data_root))
-        self.statedir = expand(testdata.get('statedir', self.data_root))
+        self.inputdir = expand(testdata.get("inputdir", self.data_root))
+        self.outputdir = expand(testdata.get("outputdir", self.data_root))
+        self.statedir = expand(testdata.get("statedir", self.data_root))
 
         # Insert global supported_extra_args
-        if 'global' in testdata:
-            if 'args' in testdata['global']:
-                args = testdata['global']['args']
+        if "global" in testdata:
+            if "args" in testdata["global"]:
+                args = testdata["global"]["args"]
                 if not isinstance(args, dict):
                     raise Exception("Global args should be a dict")
                 self.supported_extra_args = [
                     {
-                        'name': k,
-                        'description': "Global arg {}".format(k),
-                        'required': False,
-                        'default': v
+                        "name": k,
+                        "description": "Global arg {}".format(k),
+                        "required": False,
+                        "default": v,
                     }
                     for k, v in args.items()
                 ]
 
-        testdata = testdata.get('data',{})
+        testdata = testdata.get("data", {})
         for framename, details in testdata.items():
-            stages = details.get('stages',[])
-            frametype = details.get('frametype', 'pandas')
+            stages = details.get("stages", [])
+            frametype = details.get("frametype", "pandas")
 
             # We could have potentially specified multiple files that need to be loaded
             paths = []
-            transform = details['transform']
+            transform = details["transform"]
             sources = []
-            if 'source' in details:
-                source = details['source']
+            if "source" in details:
+                source = details["source"]
             else:
-                source = details['filename']
+                source = details["filename"]
             if isinstance(source, str):
                 if "inputdir" not in source:
                     source = "%(inputdir)s/%(transform)s/" + source
                 sources.append(source)
-                path = self.get_file(source, extra={
-                    'inputdir': self.inputdir,
-                    'transform': transform
-                })
+                path = self.get_file(
+                    source, extra={"inputdir": self.inputdir, "transform": transform}
+                )
                 paths.extend(glob.glob(path))
             elif isinstance(source, list):
                 for f in source:
-                    if 'inputdir' not in f:
+                    if "inputdir" not in f:
                         f = "%(inputdir)s/%(transform)s/" + f
                     sources.append(f)
-                    path = self.get_file(f, extra={
-                        'inputdir': self.inputdir,
-                        'transform': transform
-                    })
+                    path = self.get_file(
+                        f, extra={"inputdir": self.inputdir, "transform": transform}
+                    )
                     paths.extend(glob.glob(path))
             elif isinstance(source, dict):
                 for group in source:
                     for f in source[group]:
                         sources.append(f)
-                        path = self.get_file(f, extra={
-                            'transform': transform,
-                            'group': group
-                        })
+                        path = self.get_file(
+                            f, extra={"transform": transform, "group": group}
+                        )
                         paths.extend(glob.glob(path))
 
             if len(paths) == 0:
-                logger.warning("No files to load for {}".format(framename),
-                               extra={
-                                   'data': "Searched " + ",".join(sources)
-                               })
+                logger.warning(
+                    "No files to load for {}".format(framename),
+                    extra={"data": "Searched " + ",".join(sources)},
+                )
                 continue
 
             _dfs = []
@@ -555,12 +558,12 @@ class MockConfig(object):
                 if not os.path.exists(path):
                     raise Exception("Missing input file: {}".format(path))
 
-                if frametype in ['pandas']:
-                    params = details.get('params', {})
+                if frametype in ["pandas"]:
+                    params = details.get("params", {})
                     df = pd.read_csv(path, **params)
-                elif frametype in ['dict']:
+                elif frametype in ["dict"]:
                     params = {}
-                    df = json.load(open(path, 'r'))
+                    df = json.load(open(path, "r"))
                 _dfs.append(df)
 
             # => Load them ..
@@ -569,42 +572,34 @@ class MockConfig(object):
             elif len(_dfs) == 1:
                 df = _dfs[0]
             else:
-                if 'mergedf' not in details:
+                if "mergedf" not in details:
                     raise Exception("Please use 'mergedf' attribute to combine")
 
-                df = details['mergedf'](_dfs)
+                df = details["mergedf"](_dfs)
 
             details = {
-                'df': df,
-                'frametype': frametype,
-                'stages': stages,
-                'transform': details['transform'],
-                'history': [
-                    {
-                        'log': 'Test load of data'
-                    }
-                ],
-                'params': [
+                "df": df,
+                "frametype": frametype,
+                "stages": stages,
+                "transform": details["transform"],
+                "history": [{"log": "Test load of data"}],
+                "params": [
                     {
                         "type": "lineage",
                         "dependencies": [
-                            {
-                                "type": "file",
-                                "usage": "input",
-                                "name": paths
-                            }
-                        ]
+                            {"type": "file", "usage": "input", "name": paths}
+                        ],
                     }
-                    #{
+                    # {
                     #    'type': 'export',
                     #    'params': params
-                    #}
-                ]
+                    # }
+                ],
             }
             self.state.update_frame(framename, details)
 
         # => Load the runs
-        runs = testdata.get('runs',[])
+        runs = testdata.get("runs", [])
         self.runmanager.load_runs(runs, self.state)
 
     def store_test_state(self, obj, testdata=None):
@@ -618,23 +613,27 @@ class MockConfig(object):
         except:
             pass
 
-        metadata = json.dumps({
-            'name': self.name,
-            'runid': self.state.runid,
-            'notes': self.state.notes,
-            'performance': self.state.performance_notes,
-            'stats': self.state.stats,
-            'start_time': self.state.start_time,
-            'end_time': self.state.end_time,
-            'pid': self.state.pid,
-            'frames': self.state.state,
-            'expectations': self.state.expectations,
-            'accesses': self.accessed_files,
-            'versionmap': self.versionmap,
-        }, cls=CustomEncoder, indent=4)
+        metadata = json.dumps(
+            {
+                "name": self.name,
+                "runid": self.state.runid,
+                "notes": self.state.notes,
+                "performance": self.state.performance_notes,
+                "stats": self.state.stats,
+                "start_time": self.state.start_time,
+                "end_time": self.state.end_time,
+                "pid": self.state.pid,
+                "frames": self.state.state,
+                "expectations": self.state.expectations,
+                "accesses": self.accessed_files,
+                "versionmap": self.versionmap,
+            },
+            cls=CustomEncoder,
+            indent=4,
+        )
 
-        path = os.path.join(self.statedir, 'metadata.json')
-        with open(path, 'w') as fd:
+        path = os.path.join(self.statedir, "metadata.json")
+        with open(path, "w") as fd:
             fd.write(metadata)
 
         frames = self.state.get_frame_list()
@@ -643,24 +642,24 @@ class MockConfig(object):
 
             # Parameters for export
             export = {
-                'index': False,
+                "index": False,
             }
 
-            for p in detail['params']:
-                if p.get('type',None) == 'export':
-                    export.update(p.get('params', {}))
+            for p in detail["params"]:
+                if p.get("type", None) == "export":
+                    export.update(p.get("params", {}))
 
-            df = detail['df']
+            df = detail["df"]
             if df is None:
                 continue
 
-            frametype = detail['frametype']
-            if frametype in ['pandas', 'spark']:
-                path = os.path.join(self.statedir, frame + '.csv')
+            frametype = detail["frametype"]
+            if frametype in ["pandas", "spark"]:
+                path = os.path.join(self.statedir, frame + ".csv")
                 df.to_csv(path, **export)
-            elif frametype in ['dict']:
-                path = os.path.join(self.statedir, frame + '.json')
-                with open(path, 'w') as fd:
+            elif frametype in ["dict"]:
+                path = os.path.join(self.statedir, frame + ".json")
+                with open(path, "w") as fd:
                     fd.write(json.dumps(df, indent=4))
 
         return self.statedir
@@ -669,31 +668,30 @@ class MockConfig(object):
         return self.state
 
     def add_notification_targets(self, targets):
-        if 'email' in targets:
-            self.notifications['email'].extend(targets['email'])
+        if "email" in targets:
+            self.notifications["email"].extend(targets["email"])
 
     def get_credentials(self, name):
 
         siteconf = self.siteconf
 
-        if (('credentials' not in siteconf) or
-            (name not in siteconf['credentials'])):
+        if ("credentials" not in siteconf) or (name not in siteconf["credentials"]):
             raise Exception("missing credentials")
 
-        return siteconf['credentials'][name]
+        return siteconf["credentials"][name]
 
     def get_versionmap(self):
         return copy.copy(self.versionmap)
 
-class MockPipeline():
 
+class MockPipeline:
     def execute(self, transformcls, testdata, save_state=False, context={}):
 
         # => First create a mock pipeline.
         config = MockConfig(context)
 
         # => Configure it...
-        config_params = testdata.get('config', {})
+        config_params = testdata.get("config", {})
         config.configure(config_params)
 
         # => Load test state
@@ -706,21 +704,19 @@ class MockPipeline():
         transform = transformcls(config=config)
 
         # => Configure
-        conf = testdata.get('conf', {})
+        conf = testdata.get("conf", {})
         transform.configure(conf)
 
         # => Validate the configuration
-        transform.validate('conf', state)
-        transform.validate('args', state)
+        transform.validate("conf", state)
+        transform.validate("args", state)
 
         # => Now call the process
         transform.process(state)
 
         # => Now validate the output state
-        transform.validate('results', state)
+        transform.validate("results", state)
 
         # => Store the final resultant state
         if save_state:
             config.store_test_state(transform, testdata)
-
-
