@@ -2,4 +2,4 @@ from django.apps import AppConfig
 
 
 class APPNAMEConfig(AppConfig):
-    name = 'APPNAME'
+    name = "APPNAME"
